@@ -8,7 +8,6 @@
 #' @param rot     rotation: 0 (default), 90, 180 (survival), or 270.
 #' @param thC0k   vector of copula parameters
 #' @param dfC     degrees of freedom for the Student copula (default is NULL)
-#' @param adj     tuning parameter (>= 1) that can be used to prevent overflow when the cluster size n is very large; when  n<=100 OR Bernoulli marginals, no adjustment is required; when n>=500 for the Poisson likelihood fails due to overflow problem;  adj=3 prevents this in 100\% cases
 #' @param nq      number of nodes and weighted for Gaussian quadrature of the product of conditional copulas; default is 31.
 #' @return \item{condmed}{Conditional a posteriori median.}
 #'
@@ -24,7 +23,7 @@
 
 
 
-MAP.discrete = function(vv,uu,family,rot,thC0k,dfC=NULL,adj=1,nq=35){
+MAP.discrete = function(vv,uu,family,rot,thC0k,dfC=NULL,nq=35){
   d=length(vv)
   gl=statmod::gauss.quad.prob(nq)
   nl=gl$nodes
@@ -42,8 +41,14 @@ MAP.discrete = function(vv,uu,family,rot,thC0k,dfC=NULL,adj=1,nq=35){
 
 
   pdf=matrix(tem-tem1,ncol=nq)
-  den=matrixStats::colProds(adj*pdf)
-  normC=sum(wl*den)
+  #den=matrixStats::colProds(adj*pdf)
+
+  lden=colSums(log(pdf))
+  lmax = max(lden)
+  wprd = wl*exp(lden - lmax)
+  normC = sum(wprd)
+
+  #normC=sum(wl*den)
   cdff=function(x,thx){
     nnx=x*nn
 
@@ -51,8 +56,14 @@ MAP.discrete = function(vv,uu,family,rot,thC0k,dfC=NULL,adj=1,nq=35){
     temx1=pcond(uuu,nnx,family,rot,param,dfC)
 
     pdfx=matrix(temx-temx1,ncol=nq)
-    denx=matrixStats::colProds(adj*pdfx)
-    cdfx=x*sum(wl*denx)/normC
+    #denx=matrixStats::colProds(adj*pdfx)
+
+    lpdfx=colSums(log(pdfx))
+    wprd = wl*exp(lpdfx - lmax)
+    cdfx = x*sum(wprd)/normC
+
+    #denx=matrixStats::colProds(pdfx)
+    #cdfx=x*sum(wl*denx)/normC
     return(cdfx)
   }
   condmed=invfunc(0.5,cdff,0,lb=1e-10,ub=1-1e-10,tol=1e-8)
